@@ -2,7 +2,8 @@
 # -*- sh -*-
 # vim:ft=sh:ts=8:sw=4:noet
 #
-# Suspend Script - Copyright (C) 2004 Bernard Blackham <bernard@blackham.com.au>
+# Hibernate Script
+# Copyright (C) 2004-2006 Bernard Blackham <bernard@blackham.com.au>
 #
 # The hibernate-script package is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as published by
@@ -19,20 +20,48 @@
 # Place - Suite 330, Boston, MA 02111-1307, USA.
 #
 
+umask 077
+
+USING_ZSH=
+USING_BASH=
+NEED_POSIX=
+
 # For zsh sanity...
 #   allows splitting strings on whitespace in zsh.
-setopt SH_WORD_SPLIT 2>/dev/null || true
+setopt SH_WORD_SPLIT 2>/dev/null && USING_ZSH=1 || true
 #   allows sourced files to know they're sourced in zsh.
 unsetopt FUNCTION_ARGZERO 2>/dev/null || true
 
+# Detect bash and tell it not to be POSIX, because we like it better that way.
+shopt > /dev/null 2>&1 && USING_BASH=1
+[ -n "$USING_BASH" ] && set +o posix 2>/dev/null
+
+[ -z "$USING_BASH$USING_ZSH" ] && NEED_POSIX=1
+
+# We prefer to use dash if we've got it.
+if [ -z "$NEED_POSIX" ] && [ -z "$TRIED_DASH" ] &&
+	command -v dash > /dev/null 2>&1 ; then
+
+    if [ "${0##*/}" = "$0" ] ; then
+	myself=`command -v $0` || myself=
+    elif [ -x "$0" ] ; then
+	myself=$0
+    fi
+
+    TRIED_DASH=1
+    export TRIED_DASH
+    [ -n "$myself" ] && exec dash $myself $*
+fi
+unset TRIED_DASH
+
 SWSUSP_D="/etc/hibernate"
 SCRIPTLET_PATH="$SWSUSP_D/scriptlets.d /usr/local/share/hibernate/scriptlets.d /usr/share/hibernate/scriptlets.d"
-CONFIG_FILE="$SWSUSP_D/hibernate.conf"
-EXE=`basename $0`
-VERSION="1.03"
+DEFAULT_CONFIG_FILE="$SWSUSP_D/hibernate.conf"
+EXE="${0##*/}"
+VERSION="2.0"
 
 # Add these to the $PATH just in case.
-PATH="$PATH:/sbin:/usr/sbin"
+PATH="$PATH:/sbin:/usr/sbin:/usr/local/sbin"
 export PATH
 
 # vecho N <echo params>: acts like echo but with verbosity control - If it's
@@ -42,6 +71,7 @@ vecho() {
     local v
     v="$1"
     shift
+    [ "x$LOG_TIMESTAMP" = "x1" ] && set -- $(date "+%b %e %H:%M:%S.%2N") "$@"
     if [ "$v" -le $VERBOSITY ] ; then
 	echo $@
     else
@@ -59,10 +89,12 @@ vcat() {
     v="$1"
     shift
     if [ "$v" -le $VERBOSITY ] ; then
-	cat $@
+	cat "$@"
     else
 	if [ "$v" -le $LOG_VERBOSITY -a "$LOGPIPE" != "cat" ] ; then
-	    cat $@ | $LOGPIPE > /dev/null
+	    cat "$@" | $LOGPIPE > /dev/null
+	else
+	    cat > /dev/null
 	fi
     fi
 }
@@ -86,6 +118,13 @@ AddResumeHook() {
 }
 RESUME_BITS=""
 
+# AddTerminationHandler <function name>: adds the given function to the chain of
+# functions to handle termination cleanups.
+AddTerminationHandler() {
+    TERMINATION_HANDLERS="$TERMINATIONS_HANDLERS $1"
+}
+TERMINATION_HANDLERS=""
+
 # AddConfigHandler <function name>: adds the given function to the chain of
 # functions to handle extra configuration options.
 AddConfigHandler() {
@@ -135,23 +174,95 @@ $WRAPPED_HELP
 }
 CMDLINE_OPTIONS_HELP=""
 
-# AddConfigHelp <item name> <item help>: Adds an option to the option help
-# text. <item help> must only contain line breaks if a new paragraph really
-# does want to be started. Text wrapping is taken care of.
+# AddConfigHelp <item name> <item help>: Deprecated. We no longer write
+# config help in --help
 AddConfigHelp() {
-    [ -n "$DISABLE_HELP" ] && return
-    local ATEXT
-    local WRAPPED_HELP
-    ATEXT="  $1"
-    [ -n "$CURRENT_SOURCED_SCRIPTLET" ] && ATEXT=`LeftRightPadText "$ATEXT" "[$CURRENT_SOURCED_SCRIPTLET]"`
-    WRAPPED_HELP="`echo \"$2\" | WrapHelpText`"
-    ATEXT="$ATEXT
-$WRAPPED_HELP
-
-"
-    CONFIGURATION_OPTIONS_HELP="$CONFIGURATION_OPTIONS_HELP$ATEXT"
+    return 0
+}
+
+# FindXServer: Tries to find a running X server on the system. If one is found,
+# will set DISPLAY, XAUTHORITY and the XUSER variable, and return 0.
+# Otherwise, returns 1.
+FindXServer() {
+    [ -n "$FIND_X_SERVER_RESULT" ] && return $FIND_X_SERVER_RESULT
+
+    [ -z "$DISPLAY" ] && DISPLAY=:0
+    export DISPLAY
+
+    # See if we need to authenticate to this X server.
+    PATH=$PATH:/usr/bin/X11:/usr/X11R6/bin
+    export PATH
+    xhost=`command -v xhost 2>/dev/null`
+    if [ $? -ne 0 ] ; then
+	vecho 0 "$EXE: Could not find xhost program. Disabling X scriptlets."
+	FIND_X_SERVER_RESULT=1
+	return 1
+    fi
+
+    # Find a useful XAUTHORITY and ideally a username too if we can!
+    local xuser xauth xpid
+    for display in $(seq 0 8); do
+	xuser=$(last ":$display" | grep 'still logged in' | awk '{print $1}') || true
+	if [ -n "$xuser" ]; then
+	    xhome=$(getent passwd "$xuser" | cut -d: -f 6)
+	    if [ -n "$xhome" ]; then
+		xauth=$xhome/.Xauthority
+	    fi
+	fi
+	if [ -z "$xhome" ] || [ -z "$xauth" ]; then
+	    xauth=
+	    xuser=
+	    xhome=
+	else 
+	    break
+	fi
+    done
+    if [ -z $xuser ] ; then
+        for xpid in `pidof kwrapper ksmserver kdeinit gnome-session fvwm fvwm2 pwm blackbox fluxbox WindowMaker X XFree86 Xorg` ; do
+	    # Ensure the process still exists, and we aren't hallucinating.
+	    [ -d "/proc/$xpid/" ] || continue
+
+	    xauth="`get_env_var_of_process $xpid XAUTHORITY`"
+	    xhome="`get_env_var_of_process $xpid HOME`"
+	    xuser=`/bin/ls -ld /proc/$xpid/ | awk '{print $3}'`
+	    [ -z $xauth ] && [ -n $xhome ] && [ -f $xhome/.Xauthority ] && xauth=$xhome/.Xauthority
+
+	    [ -z $xauth ] && continue
+
+	    XAUTHORITY=$xauth su $xuser -c "$xhost" > /dev/null 2>&1 && break
+
+	   xauth=
+	   xuser=
+       done
+    fi
+    if [ -n $xuser ] ; then
+	XUSER=$xuser
+	if [ -n "$xauth" ] ; then
+	    XAUTHORITY=$xauth
+	    export XAUTHORITY
+	fi
+    else
+	FIND_X_SERVER_RESULT=1
+	vecho 0 "$EXE: Could not attach to X server on $DISPLAY. Disabling X scriptlets."
+	return 1
+    fi
+
+    FIND_X_SERVER_RESULT=0
+    return 0
+}
+
+# UsingSuspendMethod <method name>: is called when a suspend method is chosen.
+# If a suspend method has already been chosen, the hibernate script will be
+# terminated with an error.
+UsingSuspendMethod() {
+    if [ -n "$HIBERNATE_SUSPEND_METHOD" ] ; then
+	vecho 0 "$EXE: More than one suspend method has been chosen."
+	vecho 0 "Please edit your config file so only one method is used."
+	exit 1
+    fi
+    HIBERNATE_SUSPEND_METHOD=$1
+    return 0
 }
-CONFIGURATION_OPTIONS_HELP=""
 
 ##############################################################################
 ### Helper functions                                                       ###
@@ -224,6 +335,16 @@ BEGIN {
 
 }
 
+# PluginTermination <params>: queries all loaded scriptlets if they want to
+# handle the given event. Returns 0 if the option was handled, 1 otherwise.
+PluginTermination() {
+    local i
+    for i in $TERMINATION_HANDLERS ; do
+	$i $@ && return 0
+    done
+    return 1
+}
+
 # PluginConfigOption <params>: queries all loaded scriptlets if they want to
 # handle the given option. Returns 0 if the option was handled, 1 otherwise.
 PluginConfigOption() {
@@ -238,14 +359,14 @@ PluginConfigOption() {
 # the script. It exits the script if we don't.
 EnsureHavePrerequisites() {
     local i
-    for i in awk grep sort getopt basename ; do
-	if ! which $i > /dev/null; then
-	    echo "Could not find required program \"$i\". Aborting."
+    for i in awk grep sort getopt basename chvt ; do
+	if ! command -v $i > /dev/null; then
+	    vecho 0 "Could not find required program \"$i\". Aborting."
 	    exit 1
 	fi
     done
     # Improvise printf using awk if need be.
-    if ! which printf > /dev/null 2>&1 ; then
+    if ! command -v printf > /dev/null 2>&1 ; then
 	# This implementation fails on strings that contain double quotes.
 	# It does the job for the help screen at least.
 	printf() {
@@ -261,17 +382,27 @@ EnsureHavePrerequisites() {
 	}
     fi
     # Improvise mktemp in case we need it too!
-    if ! which mktemp > /dev/null 2>&1 ; then
+    if ! command -v mktemp > /dev/null 2>&1 ; then
 	# Use a relatively safe equivalent of mktemp. Still suspectible to race
 	# conditions, but highly unlikely.
 	mktemp() {
 	    local CNT
 	    local D
 	    local FN
+	    local RND1
+	    local RND2
 	    CNT=1
-	    while :; do
+	    if [ -z "$RANDOM" ] ; then
+	    	# A fix for shells that do not have this bash feature
+		RND1=$(dd if=/dev/urandom count=1 2>/dev/null|cksum|cut -c"1-5")
+		RND2=$(dd if=/dev/urandom count=1 2>/dev/null|cksum|cut -c"1-5")
+	    else
+	        RND1=$RANDOM
+		RND2=$RANDOM
+	    fi
+	    while true ; do
 		D=`date +%s`
-		FN=/tmp/swsusptemp-$$$D$RANDOM$RANDOM$CNT
+		FN=/tmp/tmp.hibernate.$$$D$RND1$RND2$CNT
 		[ -f $FN ] && continue
 		touch $FN && break
 		CNT=$(($CNT+1))
@@ -289,10 +420,9 @@ Usage: $EXE [options]
 Activates software suspend and control its parameters.
 
 $CMDLINE_OPTIONS_HELP
-The following config file options are available (module name in brackets):
+For configuration file options for hibernate.conf, man hibernate.conf.
 
-$CONFIGURATION_OPTIONS_HELP
-Hibernate Script $VERSION                           (C) 2004 Bernard Blackham
+Hibernate Script $VERSION                      (C) 2004-2006 Bernard Blackham
 EOT
     return
 }
@@ -332,7 +462,7 @@ DoGetOpt() {
 	    -v|--verbosity)
 		OPT_VERBOSITY="${1#\'}"
 		OPT_VERBOSITY="${OPT_VERBOSITY%\'}"
-		VERBOSITY="$OPT_VERBOSITY"
+		EnsureNumeric "verbosity" $OPT_VERBOSITY && VERBOSITY="$OPT_VERBOSITY"
 		shift
 		;;
 	    -q)
@@ -362,21 +492,25 @@ DoGetOpt() {
 # ParseOptions <options>: process all the command-line options given
 ParseOptions() {
     local opts
-    opts="`getopt -n \"$EXE\" -o \"Vhfksv:nqF:$EXTRA_SHORT_OPTS\" -l \"help,force,kill,verbosity:,dry-run,config-file:$EXTRA_LONG_OPTS\" -- \"$@\"`" || exit 1
+    opts="`getopt -n \"$EXE\" -o \"Vhfksv:nqF:$EXTRA_SHORT_OPTS\" -l \"help,force,kill,verbosity:,dry-run,config-file:,version$EXTRA_LONG_OPTS\" -- \"$@\"`" || exit 1
     DoGetOpt $opts
 }
 
 # CheckImplicitAlternateConfig <$0> : checks $0 to see if we should be
 # implicitly using a different configuration file.
 CheckImplicitAlternateConfig() {
+    # Don't find one if we've already been specified one.
+    [ -n "$CONFIG_FILE" ] && return 0
+
     local self
-    self=`basename $0`
+    self="${0##*/}"
     case $self in
 	hibernate-*) 
 	    CONFIG_FILE="$SWSUSP_D/${self#hibernate-}.conf"
-	    vecho 1 "Using implicit configuration file $CONFIG_FILE"
+	    vecho 3 "$EXE: Using implicit configuration file $CONFIG_FILE"
 	    ;;
 	*)
+	    CONFIG_FILE="${DEFAULT_CONFIG_FILE}"
 	    ;;
     esac
     return 0
@@ -386,17 +520,23 @@ CheckImplicitAlternateConfig() {
 # be dealt with before scriptlets and before other command-line options.
 PreliminaryGetopt() {
     local opt
-    local next_is_config
-    for opt in `getopt -q -o hF: -l help,config-file:,version -- "$@"` ; do
-	if [ -n "$next_is_config" ] ; then
-	    CONFIG_FILE="${opt#\'}"
-	    CONFIG_FILE="${CONFIG_FILE%\'}"
-	    next_is_config=
-	fi
+    set -- `getopt -q -o hv:F: -l help,verbosity:,config-file:,version -- "$@"`
+    while [ -n "$*" ] ; do
+	opt="$1"
+	shift
 
 	case $opt in
 	    -h|--help) HELP_ONLY=1 ;;
-	    -F|--config-file) next_is_config=1 ;;
+	    -F|--config-file)
+		CONFIG_FILE="${1#\'}"
+		CONFIG_FILE="${CONFIG_FILE%\'}"
+		;;
+	    -v|--verbosity)
+		OPT_VERBOSITY="${1#\'}"
+		OPT_VERBOSITY="${OPT_VERBOSITY%\'}"
+		EnsureNumeric "verbosity" $OPT_VERBOSITY && VERBOSITY="$OPT_VERBOSITY"
+		shift
+		;;
 	    --version)
 		echo "Hibernate Script $VERSION"
 		exit 0
@@ -423,7 +563,7 @@ LoadScriptlets() {
 	    case "$scriptlet" in *~|*.bak) continue ;; esac
 
 	    # Don't source a scriptlet by name more than once.
-	    scriptlet_name="`basename $scriptlet`"
+	    scriptlet_name="${scriptlet##*/}"
 
 	    eval "prev_path=\"\${HAVE_SOURCED_SCRIPTLET_$scriptlet_name}\""
 
@@ -467,7 +607,18 @@ BoolIsOn() {
     exit 1
 }
 
-# ProcessConfigOption: takes a configuration option and it's parameters and
+IsANumber() {
+    case "$1" in *[!0-9]*|"") return 1 ;; esac
+    return 0
+}
+
+EnsureNumeric() {
+    IsANumber "$2" && return 0
+    echo "$EXE: Invalid numeric value ($2) for option $1 in configuration file"
+    exit 1
+}
+
+# ProcessConfigOption: takes a configuration option and its parameters and
 # passes it out to the relevant scriptlet.
 ProcessConfigOption() {
     local option
@@ -488,24 +639,56 @@ ProcessConfigOption() {
 	    [ -z "$LOGFILE" ] &&
 		LOGFILE="$params"
 	    ;;
-	logverbosity)
-	    LOG_VERBOSITY="$params"
+	logtimestamp)
+	    if BoolIsOn "$option" "$params" ; then
+		LOG_TIMESTAMP=1
+	    else
+		LOG_TIMESTAMP=0
+	    fi
 	    ;;
-	swsuspvt)
-	    SWSUSPVT="$params"
+	logverbosity)
+	    EnsureNumeric "$option" "$params" && LOG_VERBOSITY="$params"
+	    ;;
+	swsuspvt|hibernatevt)
+	    EnsureNumeric "$option" "$params" && SWSUSPVT="$params"
 	    ;;
 	verbosity)
-	    [ -z "$OPT_VERBOSITY" ] &&
+	    [ -z "$OPT_VERBOSITY" ] && EnsureNumeric "$option" "$params" &&
 		VERBOSITY="$params"
 	    ;;
 	distribution)
-	    [ -z "$DISTRIBUTION" ] &&
-		DISTRIBUTION="$params"
+	    [ -z "$DISTRIBUTION" ] && DISTRIBUTION=`echo $params | tr '[A-Z]' '[a-z]'`
+	    ;;
+	xdisplay)
+	    DISPLAY="$params"
+	    export DISPLAY
+	    ;;
+        include)
+	    vecho 3 "$EXE: Including configuration from $params"
+	    if ! ReadConfigFile "$params" ; then
+		echo "$EXE: Unable to read configuration file $params (from Include directive)."
+		exit 1
+	    fi
+	    ;;
+	trymethod)
+	    if [ -z "$HIBERNATE_SUSPEND_METHOD" ] ; then
+		NO_COMPLAIN_UNSUPPORTED=1
+		vecho 1 "$EXE: Trying method in $1..."
+		if ! ReadConfigFile "$params" ; then
+		    echo "$EXE: Unable to read configuration file $params (from TryMethod directive)."
+		fi
+		NO_COMPLAIN_UNSUPPORTED=
+	    fi
 	    ;;
 	*)
 	    if ! PluginConfigOption $option $params ; then
-		echo "$EXE: Unknown configuration option ($option)"
-		exit 1
+		# See if we're trying something new, and haven't yet got a suspend method
+		if [ -n "$NO_COMPLAIN_UNSUPPORTED" ] && [ -z "$HIBERNATE_SUSPEND_METHOD" ] ; then
+		    return 1
+		else
+		    echo "$EXE: Unknown configuration option ($option)"
+		    exit 1
+		fi
 	    fi
 	    ;;
     esac
@@ -514,22 +697,36 @@ ProcessConfigOption() {
 
 # ReadConfigFile: reads in a configuration file from stdin and sets the
 # appropriate variables in the script. Returns 0 on success, exits on errors
+conffiles_read=
 ReadConfigFile() {
     local option params
-    if [ ! -f "$CONFIG_FILE" ] ; then
-	echo "WARNING: No configuration file found ($CONFIG_FILE)."
-	echo "This script probably won't do anything."
-	return 0
+    local file_name; file_name="$1"
+    
+    if [ ! -f "${file_name}" ] ; then
+	# Search in /etc/hibernate
+	if [ -f "$SWSUSP_D/$file_name" ] ; then
+	    file_name="$SWSUSP_D/$file_name"
+	else
+	    echo "WARNING: No configuration file found (${file_name})."
+	    echo "This script probably won't do anything."
+	    return 1
+	fi
     fi
-    while : ; do
+
+    if [ "${conffiles_read#*$file_name}" != "$conffiles_read" ]; then
+  	vecho 2 "$EXE: Skipping already included config file $file_name"
+  	return 0
+    fi
+
+    while true ; do
 	# Doing the read this way allows means we don't require a new-line
 	# at the end of the file.
 	read option params
 	[ $? -ne 0 ] && [ -z "$option" ] && break
 	[ -z "$option" ] && continue
 	case $option in ""|\#*) continue ;; esac # avoids a function call (big speed hit)
-	ProcessConfigOption $option $params
-    done < $CONFIG_FILE
+	ProcessConfigOption $option $params || break
+    done < ${file_name}
     return 0
 }
 
@@ -543,7 +740,7 @@ AddInbuiltHelp() {
     AddOptionHelp "-F<file>, --config-file=<file>" "Use the given configuration file instead of the default ($CONFIG_FILE)"
     AddOptionHelp "--dry-run" "Don't actually do anything."
 
-    AddConfigHelp "SwsuspVT N" "If specified, output from the suspend script is redirected to the given VT instead of stdout."
+    AddConfigHelp "HibernateVT N" "If specified, output from the suspend script is redirected to the given VT instead of stdout."
     AddConfigHelp "Verbosity N" "Determines how verbose the output from the suspend script should be:
    0: silent except for errors
    1: print steps
@@ -551,10 +748,13 @@ AddInbuiltHelp() {
    3: print steps in lots of detail
    4: print out every command executed (uses -x)"
     AddConfigHelp "LogFile <filename>" "If specified, output from the suspend script will also be redirected to this file - useful for debugging purposes."
+    AddConfigHelp "LogTimestamp <boolean>" "If logging to file, will place timestamps on each log entry. This is only recommended if you are trying to optimise suspend/resume time, otherwise it will just slow the process down."
     AddConfigHelp "LogVerbosity N" "Same as Verbosity, but controls what is written to the logfile."
     AddConfigHelp "AlwaysForce <boolean>" "If set to yes, the script will always run as if --force had been passed."
     AddConfigHelp "AlwaysKill <boolean>" "If set to yes, the script will always run as if --kill had been passed."
     AddConfigHelp "Distribution <debian|fedora|mandrake|redhat|gentoo|suse|slackware>" "If specified, tweaks some scriptlets to be more integrated with the given distribution."
+    AddConfigHelp "Include <filename>" "Immediately include configuration directives from the given file."
+    AddConfigHelp "XDisplay <display location>" "Specifies where scriptlets that use the X server should find one. (Default: :0)"
 }
 
 EnsureHaveRoot() {
@@ -569,7 +769,7 @@ EnsureHaveRoot() {
 # it easy to decide whether or not to pipe its output to $LOGPIPE or not.
 DoWork() {
     # Trap Ctrl+C
-    trap ctrlc_handler INT
+    trap ctrlc_handler INT HUP
 
     # Do everything we need to do to suspend. If anything fails, we don't
     # suspend.  Suspend itself should be the last one in the sequence.
@@ -605,7 +805,7 @@ DoWork() {
 	    break
 	fi
 	if [ -n "$SUSPEND_ABORT" ] ; then
-	    vecho 0 "$EXE: Aborted suspend with Ctrl+C."
+	    vecho 0 "$EXE: Suspend aborted by user."
 	    EXIT_CODE=3
 	    break
 	fi
@@ -628,21 +828,30 @@ ctrlc_handler() {
     SUSPEND_ABORT=1
 }
 
+# Gets the value of the environment variable in the environment space of the
+# processes identified by its PID.
+get_env_var_of_process()
+{
+    local pid; pid="$1"
+    local envvar; envvar="$2"
+    tr '\0' '\n' </proc/$pid/environ | sed -ne "s/^$envvar=//p"
+}
+
 ############################### MAIN #########################################
 
 # Some starting values:
 VERBOSITY=0
 LOG_VERBOSITY=1
 LOGPIPE="cat"
+ERROR_TEXT=""
 
 EnsureHavePrerequisites
-EnsureHaveRoot
+
+# Test for options that will affect future choices
+PreliminaryGetopt "$@"
 
 CheckImplicitAlternateConfig $0
 
-# Test for options that will affect future choices (-h and -F currently)
-PreliminaryGetopt "$@"
-
 # Generating help text is slow. Avoid it if we can.
 if [ -n "$HELP_ONLY" ] ; then
     AddInbuiltHelp
@@ -652,8 +861,16 @@ if [ -n "$HELP_ONLY" ] ; then
 fi
 DISABLE_HELP=1
 
+EnsureHaveRoot
 LoadScriptlets
-ReadConfigFile
+ReadConfigFile "${CONFIG_FILE}"
+# Verify we actually got a suspend method we could use.
+if [ -z "$HIBERNATE_SUSPEND_METHOD" ] ; then
+    echo "$EXE: No suitable suspend methods were found on your machine."
+    echo "$EXE: You need to install a kernel with support for suspending to"
+    echo "$EXE: disk or RAM and reboot, then try again."
+    exit 1
+fi
 ParseOptions "$@"
 
 # Set a logfile if we need one.
@@ -676,12 +893,11 @@ if [ "$LOGPIPE" = "cat" ] ; then
 else
     # Sigh. Our portable way to obtain exit codes has issues in bash, so if
     # we're using bash, we do it the not-so-portable way :)
-    if shopt > /dev/null 2>&1 ; then
-	# we're using something like bash hopefully! :)
+    if [ -n "$USING_BASH" ] ; then
 	DoWork | $LOGPIPE
 	eval 'EXIT_CODE=${PIPESTATUS[0]}'
     else
-	# Evilness requires to pass the exit code back to us in a pipe.
+	# Evilness required to pass the exit code back to us in a pipe.
 	trap "" INT
 	exec 3>&1
 	eval `
@@ -693,8 +909,10 @@ else
     fi
 fi
 
+PluginTermination
+
 echo "Resumed at "`date` | $LOGPIPE > /dev/null
 
 exit $EXIT_CODE
 
-# $Id: hibernate.sh 628 2005-01-04 17:48:04Z dagobah $
+# $Id: hibernate.sh 1220 2009-04-01 02:06:57Z nigel $
